package dod;

import java.awt.Component;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.Socket;
import java.net.SocketException;

import javax.swing.JPanel;

public class ClientLogic implements Runnable {
	
	 private PrintWriter output;
	 private BufferedReader input;
	 private ClientGUI clientGUI;
	 private Thread thread;
	 private Socket server;
	 private ClientServerController CSC;
	 private boolean hasLantern = false;
	 private char[][] currentLookReply;
	 private GamePanel GamePanel;
	
	public ClientLogic(String Name, int Port, ClientGUI clientGUI) throws IOException{
		server = new Socket(Name, Port);
        output = new PrintWriter(server.getOutputStream(), false);
        input = new BufferedReader(new InputStreamReader(server.getInputStream()));
        CSC = new ClientServerController();
        this.clientGUI = clientGUI;
        // start thread
        thread = new Thread(this);
        thread.start(); 
	}

	@Override
	public void run() {
		try {
			waitforserver();
		} catch (SocketException e1) {
			CSC.ClientToClientMessage(clientGUI, "Connection lost to server");
		}
		catch (IOException e) {
			CSC.ClientToClientMessage(clientGUI, "Connection lost to server");
		} 
		catch (RuntimeException e2) {
			CSC.ClientToClientMessage(clientGUI, "Connection lost to server");
		}
	}

	private void waitforserver() throws IOException, SocketException, RuntimeException {
		while(true){
			// someone has won the game so you need to finish
			String ServerReply = input.readLine();	
			System.out.println(ServerReply);
			if (ServerReply.equals("LOSE")){
				CSC.ClientToClientMessage(clientGUI, "You Have lost, game is now disconnecting");
				System.exit(0);
			}
			// some has changed something within side your look reply
			else if (ServerReply.equals("CHANGE")){
				CSC.ClientToClientMessage(clientGUI, ServerReply);  
				LOOK();
				String temp = "";
				if (hasLantern == true) { // used has lantern
					int ForVal =8;
					int ForValMajor = 8;
					for (int line = 0; line < ForValMajor ; line++) {
						String Value = input.readLine();
						if (Value.equals("STARTTURN")){ // if it happend to be your turn when a change occurs
							ForVal = 7; // set it to print 1 less line
							ForValMajor = 9; // for for 1 more line in main loop body
						}
						else{
						temp = temp + Value + ","; // value from look
						}
					}
					// output to terminal
					String Lines[] = temp.split(",");
					for (int line = 0; line < ForVal; line++) {
						CSC.ClientToClientMessage(clientGUI, Lines[line]);
					}
					handleLookReply(temp.split(",")); // send to char[][]
				} else {
					int ForVal =6;
					int ForValMajor = 6;
					for (int line = 0; line < ForValMajor; line++) {
						String Value = input.readLine();
						if (Value.equals("STARTTURN")){
							ForVal =5;
							ForValMajor = 7;
						}
						else{
						temp = temp + Value + ",";
						}
					}
					//output to terminal
					String Lines[] = temp.split(",");
					for (int line = 0; line < ForVal; line++) {
						CSC.ClientToClientMessage(clientGUI, Lines[line]);
					}
					handleLookReplyNorm(temp.split(","));
				}
				GamePanel.update(currentLookReply);
			}
			else if (ServerReply.equals("LOOKREPLY")){
				String temp = "";
				if (hasLantern == true) {
					for (int line = 0; line < 7; line++) {
						temp = temp + input.readLine() + ",";
					}
					// output to terminal
					String Lines[] = temp.split(",");
					for (int line = 0; line < 7; line++) {
						CSC.ClientToClientMessage(clientGUI, Lines[line]);
					}
					handleLookReply(temp.split(","));
				} else {
					for (int line = 0; line < 5; line++) {
						temp = temp + input.readLine() + ",";
					}
					//output to terminal
					String Lines[] = temp.split(",");
					for (int line = 0; line < 5; line++) {
						CSC.ClientToClientMessage(clientGUI, Lines[line]);
					}
					handleLookReply(temp.split(","));
				}
			}
			else if (ServerReply.equals("CHAT")){
				String temp = input.readLine();
				CSC.ClientToClientMessageChat(clientGUI, temp);
			}
			// just print what was sent by the server
			else{
                CSC.ClientToClientMessage(clientGUI, ServerReply);  
			}
		}
	}
	
	private void sendToServer(String message){
			CSC.ClientToServerMessage(output, message);
	}

	public void MOVEN() {
		sendToServer("MOVE N");
		LOOK();
		GamePanel.update(currentLookReply);
	}

	public void MOVEW() {
		sendToServer("MOVE W");
		LOOK();
		GamePanel.update(currentLookReply);
	}

	public void MOVEE() {
		sendToServer("MOVE E");
		LOOK();
		GamePanel.update(currentLookReply);
	}

	public void MOVES() {
		sendToServer("MOVE S");
		LOOK();
		GamePanel.update(currentLookReply);
	}

	public void PICKUP() {
		// if beneath you is an L then set lantern to true
        if (getCentralSquare() == 'L'){
            hasLantern = true;
            // CREATE A NEW PANE WINDOW with lantern sizes
            GamePanel = new GamePanel(7, 7);
            // set the GUI to this Panel
            clientGUI.SCROLLGAME.setViewportView(GamePanel);
        }
		sendToServer("PICKUP");
        try {
			Thread.sleep(500);
		} catch (InterruptedException e1) {
			System.out.println("Thread Interrupted");
		}
        LOOK();
        GamePanel.update(currentLookReply);
        LanternChange();
	}

	public void HELLO(String inputMessage) {
		// The Name is a set of characters 
    	if (inputMessage.equals(" ") | inputMessage.equals("") | inputMessage == null ){
    		CSC.ClientToClientMessage(clientGUI, "You Need to enter a Name First");
    	}
    	else{
    		sendToServer("HELLO " + inputMessage);
    	}
	}

	public void LOOK() {
		this.currentLookReply = null;
		// Have a look at the map
		sendToServer("LOOK");
		 try { // Let the server reply come back so just wait a bit
				Thread.sleep(500);
		} catch (InterruptedException e1) {
					e1.printStackTrace();
		}
	}

	public void SENDCHAT(String inputMessage) {
		sendToServer("CHAT " + inputMessage);
		try {
			Thread.sleep(500); // let server reply come back
		} catch (InterruptedException e1) {
			 System.out.println("Thread Interrupted");
		}
	}

	public void ATTACKN() {
		sendToServer("ATTACK N");
		LOOK();
	}

	public void ATTACKW() {
		sendToServer("ATTACK W");
		LOOK();
	}

	public void ATTACKE() {
		sendToServer("ATTACK E");
		LOOK();
	}

	public void ATTACKS() {
		sendToServer("ATTACK S");
		LOOK();
	}

	public void ENDTURN() {
		sendToServer("ENDTURN");
	}

	public void SHOUT(String inputMessage) {
    	if (inputMessage.equals(" ") | inputMessage.equals("") | inputMessage == null ){
    		CSC.ClientToClientMessage(clientGUI, "You Need to enter a Message First");
    	}
    	else{
    		sendToServer("SHOUT" + " " + inputMessage);
    	}
	}

	public JPanel createPane() throws InterruptedException, IOException{
       if (hasLantern == true) { // use larger GamePanel
    	   return GamePanel = new GamePanel(7, 7);
       }
       else { // Use Smaller GamePanel
    	   this.GamePanel = new GamePanel(5, 5);
    	   return GamePanel; 
       }
    }
	
    /**
     * Gets the square the the user is currently on in the look reply
     * 
     * @return the square the user is standing on
     */
    private char getCentralSquare() {
    	// Return the square with 0 offset
    	return getSquareWithOffset(0, 0);
    }
    
    /**
     * Obtains a square in of the LOOKREPLY with an offset to the player
     * 
     * @return the square corresponding to the player and offset
     */
    private char getSquareWithOffset(int xOffset, int yOffset) {
    	final int lookReplySize = this.currentLookReply.length;
    	final int lookReplyCentreIndex = lookReplySize / 2; // We rely on truncation
    	return this.currentLookReply[lookReplyCentreIndex + yOffset][lookReplyCentreIndex
    	                                                             + xOffset];
    }
    
    /**
     * Used by GUI to first start the process of creating and displaying the
     * GamePanel to the user by first calling look, setting up the map
     * and then starting the thread to update the map at time intervals
     */
    public void startPane(){ // Start the Game and adds the graphic to the GamePanel
    	try {
			Thread.sleep(50); // Lets the server send GOAL "number" and StartTurn before calling look
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
        LOOK(); // get lookreply and update graphic
        GamePanel.run(currentLookReply, 2000, 2000);
        GamePanel.draw();
    }
    
    /**
     * Starts the new GamePanel up again once a lantern has been picked up
     */
    public void LanternChange() { // redraws the graphic if a lantern is used
        GamePanel.run(currentLookReply, 2000, 2000);
        GamePanel.draw();
    }
    
    /**
	 * Handles the LOOKREPLY from the game
	 * 
	 * @param lines
	 *            the lines returned as part of the LOOKREPLY
	 */
   private void handleLookReply(String[] lines) {
       // Get look reply and create map, Used for a modified look reply without first string
       final int lookReplySize = lines[1].length();
       if (lines.length != lookReplySize) {
               throw new RuntimeException("FAIL: Invalid LOOKREPLY dimensions");
       }
       this.currentLookReply = new char[lookReplySize][lookReplySize];
       for (int row = 0; row < lookReplySize; row++) {
               for (int col = 0; col < lookReplySize; col++) {
                       this.currentLookReply[row][col] = lines[row].charAt(col);  
               }
       }
   }
   
   
   /**
    * @param lines - look reply in , separated format
    */
   private void handleLookReplyNorm(String[] lines) {
       // Work out what the reply is - used when "LOOKREPLY" is included
       final int lookReplySize = lines[1].length();
       if (lines.length != lookReplySize + 1) {
               throw new RuntimeException("FAIL: Invalid LOOKREPLY dimensions");
       }
       this.currentLookReply = new char[lookReplySize][lookReplySize];
       for (int row = 0; row < lookReplySize; row++) {
               for (int col = 0; col < lookReplySize; col++) {
                       this.currentLookReply[row][col] = lines[row +1].charAt(col);  
               }
       }
   }
}


