package dod;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import javax.swing.JPanel;

public class GamePanel extends JPanel {
	// Initialise all variables and a service executor to draw the image constantly 
    private final ScheduledExecutorService service = Executors.newSingleThreadScheduledExecutor();
    private BufferedImage image;
    private Graphics2D g;
    private char multi[][];
    private TileMap tileMap;
    
    /**
     * @param heightval - Height of the Map
     * @param widthval - Width of the Map
     * Sets the Size of the Panel to 300x300 as this is the max size with a lantern + 20 using
     * 40x40 tiles to represent each char in the game. Calls tile map ensure it knows what the
     * tile size and width and height of map should be
     */
    public GamePanel(int heightval, int widthval) {
        super();
        setPreferredSize(new Dimension(300, 300));
        setFocusable(true); 
        // 40 tile size , with height and width of 5x5 or 7x7 depending on lantern
        tileMap = new TileMap(40, heightval, widthval);
    }
    
    
    /**
     * @param multi - Map of chars
     * @param WIDTHVal - Height of Image
     * @param HEIGHTVal - Width of Image
     */
    public void run(char[][]  multi, int WIDTHVal, int HEIGHTVal){
    	// creates an image that can be changed and calls this 2D image g
        image = new BufferedImage(WIDTHVal, HEIGHTVal, BufferedImage.TYPE_INT_RGB);
        g = (Graphics2D) image.getGraphics();
        // tell the tile map to create image g using the map multi
        tileMap.draw(g, multi);
        this.multi = multi;
    }
    
    
    
    /**
     * @param multi - Look reply map
     * sets the map to be updated to a new map
     */
    public void update(char[][]  multi){
    	this.multi= multi;
    }
    
    
    /**
     * Will create a service executor with a fixed delay to re-draw the image incase the GUI is moved 
     * or the user decides to move their player from within the game 
     */
    public void draw(){
       service.scheduleWithFixedDelay(new Runnable(){ // new runnable executor service
        @Override
        public void run(){ // starts the loop
            tileMap.draw(g, multi); // create image
            Graphics g2 = getGraphics(); // renders the image
            g2.drawImage(image, 0, 0, null); // draw image starting in pane at 0,0 (X,Y)
            g2.dispose(); // once drawn get rid of it
        }
      }, 0, 100, TimeUnit.MILLISECONDS); // does this every 1/10th of a second
    }

}
