package dod.game.items;

public abstract class GameItem {

	/**
     * Obtains a character representing the item, used by the textual interface.
     * 
     * @return a single character
     */
	public abstract char toChar();

	public abstract int getAmount();

	public static GameItem fromChar(char ch, int Value) {
		switch (ch) {
		case 'A':
		    return new Armour(Value);
		case 'G':
		    return new Gold(Value);
		case 'H':
		    return new Health(Value);
		case 'L':
		    return new Lantern(Value);
		case 'S':
		    return new Sword(Value);
		default:
		    throw new IllegalArgumentException("Invalid tile type" + ch);
		}
	}

	/**
     * Checks if the item can be "retained" by the player, i.e., the player
     * doesn't consume it instantly (like health) but holds on to it, like
     * sword.
     * 
     * @return true if the item is retained
     */
    public abstract boolean isRetainable();

    public int lookDistanceIncrease() {
    	// Only retainable items will be able to affect the distances
    	assert isRetainable();
    	// Return zero by default
    	return 0;
    }

	public void processPickUp(GameItemConsumer player) {
		// By default, do nothing
	}

}
